const AWSLambdaCaller = require('./services/AWSLambdaCaller');
const Util = require('./util/Util');
const MediaResolver = require('./util/MediaResolver');
const DialogFlowAccessor = require('./services/DialogFlowAccessor');
const KeyMatcherAccessor = require('./services/KeyMatcherAccessor');
const RasaAccessor = require('./services/RasaAccessor');
const defaultFallbackIntent = require('./util/defaultFallbackIntent');
const RequestPreprocessor = require('./util/RequestPreprocessor');

class AsksuiteProcessingNatural {
    static get ORDERED_ACCESSORS() { return ['DIALOG_FLOW', 'RASA', 'KEY_MATCHER']; } // Robot will search for intents in this order default
    static get RASA_FIRST_ORDERED_ACCESSORS() { return ['RASA', 'DIALOG_FLOW', 'KEY_MATCHER']; } // Robot will search for intents in this order if tryRasaFirst option is set to true

    constructor() {
    };

    findIntent(request) {

        return new  Promise(async (resolve, reject) => {
            let intent = {};

            request = await RequestPreprocessor.process(request);

            const awsLambdaCaller = new AWSLambdaCaller(request.config.CORE_LAMBDA_AWS);
            const dialogFlowAccessor = new DialogFlowAccessor(awsLambdaCaller);
            const keyMatcherAccessor = new KeyMatcherAccessor(awsLambdaCaller);
            const rasaAccessor = new RasaAccessor(request.config.RASA_URL, request.config.RASA_CONFIDENCE_THRESHOLD);

            let orderOption = AsksuiteProcessingNatural.ORDERED_ACCESSORS;
            if(request.tryRasaFirst) {
                orderOption = AsksuiteProcessingNatural.RASA_FIRST_ORDERED_ACCESSORS;
            }

            request.findIntent = defaultFallbackIntent(request.companyId);

            for (const accessorName of orderOption) {
                switch (accessorName) {
                    case 'DIALOG_FLOW':
                        intent = await dialogFlowAccessor.resolveText(request);
                        break;
                    case 'KEY_MATCHER':
                        intent = await keyMatcherAccessor.resolveText(request);
                        intent.keywordTry = { try: true };
                        break;
                    case 'RASA':
                        intent = await rasaAccessor.resolveText(request);
                        break;
                }

                request.findIntent = intent;

                if (intent.intent && !Util.isNaoentendi(intent.intent)) {
                    intent.resolver = accessorName;

                    if (intent.resolver === 'KEY_MATCHER') {
                        intent.keywordTry.match = intent.intent;
                        intent.keywordTry.intent = intent.intent;
                    }

                    //FOR exit condition
                    break;
                }

            }

            resolve(intent);

        })

    }

    resolveMedia(request) {
        return new Promise (async (resolve, reject) => {
            try {
                const mediaResolver = new MediaResolver(request.companyId);
                const intent = await mediaResolver.resolve(request.media, request.intents);
                resolve(intent)
            } catch (e) {
                reject(e);
            }
        });
    }
}

module.exports = AsksuiteProcessingNatural;
